import { RemovalPolicy } from 'aws-cdk-lib';
import * as lambda from 'aws-cdk-lib/aws-lambda';
import * as dynamodb from 'aws-cdk-lib/aws-dynamodb';
import * as s3 from 'aws-cdk-lib/aws-s3';
import { Construct } from 'constructs';
/**
 * (experimental) Properties to initialize an instance of `MicroAppsAppRelease`.
 *
 * @experimental
 */
export interface MicroAppsAppReleaseProps {
    /**
     * (experimental) Name for the Lambda function.
     *
     * While this can be random, it's much easier to make it deterministic
     * so it can be computed for passing to `microapps-publish`.
     *
     * @default auto-generated
     * @experimental
     */
    readonly functionName?: string;
    /**
     * (experimental) Bucket with the static assets of the app.
     *
     * Next.js apps need access to the static assets bucket.
     *
     * @experimental
     */
    readonly staticAssetsS3Bucket: s3.IBucket;
    /**
     * (experimental) DynamoDB table for data displayed / edited in the app.
     *
     * This table is used by @pwrdrvr/microapps-datalib.
     *
     * @experimental
     */
    readonly table: dynamodb.ITable;
    /**
     * (experimental) Removal Policy to pass to assets (e.g. Lambda function).
     *
     * @experimental
     */
    readonly removalPolicy?: RemovalPolicy;
    /**
     * (deprecated) `sharp` node module Lambda Layer for Next.js image adjustments.
     *
     * @deprecated Ignored if passed, this is no longer needed
     * @example
     *
     * https://github.com/zoellner/sharp-heic-lambda-layer/pull/3
     */
    readonly sharpLayer?: lambda.ILayerVersion;
    /**
     * (experimental) NODE_ENV to set on Lambda.
     *
     * @experimental
     */
    readonly nodeEnv?: 'dev' | 'qa' | 'prod';
}
/**
 * (experimental) Represents a Release app.
 *
 * @experimental
 */
export interface IMicroAppsAppRelease {
    /**
     * (experimental) The Lambda function created.
     *
     * @experimental
     */
    readonly lambdaFunction: lambda.IFunction;
}
/**
 * (experimental) Release app for MicroApps framework.
 *
 * @experimental
 * @remarks The Release app lists apps, versions, and allows setting the default
 * version of an app.  The app is just an example of what can be done, it
 * is not feature complete for all use cases.
 */
export declare class MicroAppsAppRelease extends Construct implements IMicroAppsAppRelease {
    private _lambdaFunction;
    /**
     * (experimental) The Lambda function created.
     *
     * @experimental
     */
    get lambdaFunction(): lambda.IFunction;
    /**
     * (experimental) Lambda function, permissions, and assets used by the MicroApps Release app.
     *
     * @experimental
     */
    constructor(scope: Construct, id: string, props: MicroAppsAppReleaseProps);
}
